pub mod data;

use crypto_box::{
    aead::Aead,
    Box,
};
use salsa20;
use rand::rngs::OsRng;
use std::convert::TryInto;
use std::sync::{
    mpsc::{Receiver, Sender},
    Arc, Mutex,
};

pub fn sender_thread(
    connexion_infos: Arc<Mutex<data::ConnexionInfos>>,
    rx: Receiver<super::tun::TunData>,
    tx: Sender<Vec<u8>>,
) {
    let mut rng = OsRng;

    let secret_key = (*connexion_infos.lock().unwrap()).privkey.clone();
    let public_key = (*connexion_infos.lock().unwrap()).pubkey.clone();

    loop {
        // On reçoit un message du dispatcher coté tun
        let msg = rx.recv().unwrap();

        let nonce = crypto_box::generate_nonce(&mut rng);
    
        let mut ciphertext = Box::new(&public_key, &secret_key)
            .encrypt(&nonce, &msg.payload[..])
            .unwrap();
    
        // On ajoute le nonce au début du message (24 octets)
        let mut send_msg = nonce.clone().to_vec();
        send_msg.append(&mut ciphertext);
    
        tx.send(send_msg).unwrap();
    }
}

pub fn receiver_thread(
    connexion_infos: Arc<Mutex<data::ConnexionInfos>>,
    rx: Receiver<Vec<u8>>,
    tx: Sender<Vec<u8>>,
) {
    let secret_key = (*connexion_infos.lock().unwrap()).privkey.clone();
    let public_key = (*connexion_infos.lock().unwrap()).pubkey.clone();

    loop {
        // On reçoit un message du dispatcher coté udp
        let msg = rx.recv().unwrap();

        let nonce_msg: &[u8] = &msg[0..24];
        let nonce_msg:& [u8; 24] = nonce_msg.try_into().unwrap();

        let nonce = salsa20::XNonce::from_slice(&*nonce_msg);
        let recv_msg = &msg[24..];

        let decrypted_plaintext = Box::new(&public_key, &secret_key)
            .decrypt(&nonce, recv_msg)
            .unwrap();

        tx.send(decrypted_plaintext).unwrap();
    }
}
